import bw2data
import bw2io

from premise_gwp import add_premise_gwp


def test_implementation():
    """
    Test that the method is properly registered.
    """

    bw2data.projects.set_current("test")
    bw2io.create_default_biosphere3()

    add_premise_gwp()

    # check that the new method exists
    new_method = ("IPCC 2021", "climate change", "GWP 100a, incl. H and bio CO2")
    assert new_method in bw2data.methods

    # check that "Carbon dioxide, in air" has now a CF of -1

    co2_in_air = [
        f
        for f in bw2data.Database("biosphere3")
        if "carbon dioxide, in air" in f["name"].lower()
    ][0]["code"]

    method = bw2data.Method(new_method)

    for cf in method.load():
        if cf[0] == co2_in_air:
            assert cf[1] == -1



import bw2data as bd


def check_biosphere_database():
    ERROR = "IPCC LCIA methods for ecoinvent biosphere flows only; install base ecoinvent data"
    assert "biosphere3" in bd.databases, ERROR


def check_biosphere_version() -> tuple:
    # check for the presence of Beryllium II
    if "Beryllium II" not in [f["name"] for f in bd.Database("biosphere3")]:
        bw2io_version = (0, 8, 7)
    else:
        bw2io_version = (0, 8, 8)

    return bw2io_version



__all__ = (
    "add_premise_gwp",
    "check_biosphere_database",
)

from pathlib import Path

DATA_DIR = Path(__file__).resolve().parent / "data"

import bw2io
from bw2io import ExcelLCIAImporter

from .biosphere import check_biosphere_database, check_biosphere_version
from .version import version as __version__


def add_premise_gwp():
    check_biosphere_database()
    bw2io_version = check_biosphere_version()

    # impact methods to create
    categories_bw2io087 = {
        (
            ("IPCC 2013", "climate change", "GWP 20a, incl. H"),
            "kg CO2-Eq",
            "IPCC 2013, with CFs for hydrogen",
            "lcia_gwp_20a.xlsx",
        ),
        (
            ("IPCC 2013", "climate change", "GWP 100a, incl. H"),
            "kg CO2-Eq",
            "IPCC 2013, with CFs for hydrogen",
            "lcia_gwp_100a.xlsx",
        ),
        (
            ("IPCC 2013", "climate change", "GWP 20a, incl. H and bio CO2"),
            "kg CO2-Eq",
            "IPCC 2013, with CFs for hydrogen and biogenic CO2 flows",
            "lcia_gwp_20a_w_bio.xlsx",
        ),
        (
            ("IPCC 2013", "climate change", "GWP 100a, incl. H and bio CO2"),
            "kg CO2-Eq",
            "IPCC 2013, with CFs for hydrogen and biogenic CO2 flows",
            "lcia_gwp_100a_w_bio.xlsx",
        ),
    }

    categories_bw2io088 = {
        (
            ("IPCC 2021", "climate change", "GWP 20a, incl. H"),
            "kg CO2-Eq",
            "IPCC 2021, with CFs for hydrogen",
            "lcia_gwp2021_20a.xlsx",
        ),
        (
            ("IPCC 2021", "climate change", "GWP 100a, incl. H"),
            "kg CO2-Eq",
            "IPCC 2021, with CFs for hydrogen",
            "lcia_gwp2021_100a.xlsx",
        ),
        (
            ("IPCC 2021", "climate change", "GWP 20a, incl. H and bio CO2"),
            "kg CO2-Eq",
            "IPCC 2021, with CFs for hydrogen and biogenic CO2 flows",
            "lcia_gwp2021_20a_w_bio.xlsx",
        ),
        (
            ("IPCC 2021", "climate change", "GWP 100a, incl. H and bio CO2"),
            "kg CO2-Eq",
            "IPCC 2021, with CFs for hydrogen and biogenic CO2 flows",
            "lcia_gwp2021_100a_w_bio.xlsx",
        ),
    }

    categories = (
        categories_bw2io088 if bw2io_version >= (0, 8, 8) else categories_bw2io087
    )

    for c in categories:
        print("Adding {}".format(c[0]))
        category = ExcelLCIAImporter(
            filepath=DATA_DIR / c[-1], name=c[0], unit=c[1], description=c[2]
        )

        # apply formatting strategies
        category.apply_strategies()

        # if bw2io < 0.8.6
        if bw2io.__version__ < (0, 8, 6):
            if len(list(category.unlinked)) == 1:
                if list(category.unlinked)[0]["name"] == "Carbon dioxide, in air":
                    category.drop_unlinked()

        # check that no flow is unlinked
        assert len(list(category.unlinked)) == 0, "Unlinked flows: {}".format(
            list(category.unlinked)
        )

        # write method
        category.write_methods(overwrite=True, verbose=True)



from setuptools import setup

v_temp = {}
with open("premise_gwp/version.py") as fp:
    exec(fp.read(), v_temp)
version = ".".join((str(x) for x in v_temp["version"]))


setup(
    name="premise_gwp",
    version=version,
    packages=[
        "premise_gwp",
    ],
    author="Romain Sacchi",
    author_email="romain.sacchi@psi.ch",
    license="BSD 3-clause",
    package_data={"premise_gwp": ["data/*.xlsx", "data/*.json"]},
    install_requires=[
        "bw2io",
        "bw2data",
        "requests",
    ],
    url="https://github.com/romainsacchi/premise_gwp",
    long_description_content_type="text/markdown",
    long_description=open("README.md").read(),
    description="Import IPCC's GWP100a method, with biogenic CO2 CFs, into Brightway2",
    classifiers=[
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
